entity RegBank is
	generic
	(
		WIDTH	: integer := 4;	-- Tamanho do dado
		SIZE	: integer := 4		-- Tamanho do banco (numero de registradores = 2^M)
	);
	port
	(
		-- Entrada(s)
		CLK, WEN3	: in  bit;									-- Signals CLK e EN do MODO de entrada (in) e do TIPO bit
		A1, A2, A3	: in  bit_vector(SIZE-1 downto 0);	-- Signals A1, A2 e A3 do MODO de entrada (in) e do TIPO bit_vector
		WD3			: in  bit_vector(WIDTH-1 downto 0);	-- Signal WD3 do MODO de entrada (in) e do TIPO bit_vector de tamanho WIDTH

		-- Saida(s)
		RD1, RD2	: out bit_vector(WIDTH-1 downto 0)		-- Signals RD1 e RD2 do MODO de saida (out) e do TIPO bit_vector de tamanho WIDTH
	);
end RegBank;

architecture rtl of RegBank is

	type RegBank_type is array (2**SIZE-1 downto 0) of bit_vector(WIDTH-1 downto 0);
	signal Reg_Bank : RegBank_type;
	signal WEN3_vector : bit_vector(2**SIZE-1 downto 0);
	signal Address1, Address2, Address3 : integer range 0 to 2**SIZE-1;

begin

	Addr3: work.bin2int
		generic map(
			n => SIZE
			)
		port map(
			bin => A3,
			int => Address3
		);
		
	decoder:for i in 2**SIZE-1 downto 0 generate
		WEN3_vector(i) <= '1' when Address3 = i else '0';
	end generate;
		

	bank:for i in 2**SIZE-1 downto 0 generate
		regi: work.reg
			generic map (
				n => WIDTH
			)
			port map (
				clk => CLK,
				clr => '0',
				EN => WEN3_vector(i) and WEN3,
				d => WD3,
				q => Reg_Bank(i)
			);
	end generate;
	
	Addr1: work.bin2int
		generic map(
			n => SIZE
			)
		port map(
			bin => A1,
			int => Address1
		);
		
	Addr2: work.bin2int
		generic map(
			n => SIZE
			)
		port map(
			bin => A2,
			int => Address2
		);
		
	RD1 <= Reg_Bank(Address1);
	RD2 <= Reg_Bank(Address2);
	
	
	
end rtl;
